#include <qpainter.h>
#include <math.h>
#include <ctype.h>
#include "canvas.h"
#include "cone.h"

Canvas::Canvas( QWidget *parent, const char *name )
		: QGLWidget( parent, name )
{
	signo = 1;	
	shade = 0;

	esena = 1; 
		 
	wholight = 0;
	scale = 1;
	angle = 0.0;
	// setMouseTracking( TRUE );
	setFocusPolicy( QWidget::StrongFocus );

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	cone1 = new Cone( 0.0, 0.0, 0.0, 1.0, 0.8 );
}


void Canvas::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Canvas::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

//
// This method puts the initial coordinates each time window is resized
//
void Canvas::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if ( w <= h ) 
		glOrtho ( -1.0, 1.0, -1.0/aspect, 1.0/aspect, -1.0, 1.0);
	else
		glOrtho ( -aspect, aspect, -1.0, 1.0, -1.0, 1.0);

	emit changeSize ( );
}

QSizePolicy Canvas::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Canvas::paintGL(void)
{
	static float position0[] = {0.0, 0.0, 1.0, 1.0};
	static float position1[] = {0.0, 0.0,-1.0, 1.0};

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if ( !shade ) glShadeModel( GL_FLAT );
	else glShadeModel( GL_SMOOTH );

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */
	glRotatef (xrot, 1.0, 0.0, 0.0);
	glRotatef (yrot, 0.0, 1.0, 0.0);

	glScalef ( 0.8, 0.8, 0.8);	  /* modeling transformation */ 

	glRotatef ( angle, 1.0, 0.0, 0.0);
		 
	cone1 -> show ( );
	 
	glBegin(GL_LINES);
		// Draw x axis:
		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3f(-0.1, 0.0, 0.0);
		glVertex3f( 1.1, 0.0, 0.0);
		// Draw y axis:
		glVertex3f( 0.0, -0.1, 0.0);
		glVertex3f( 0.0,  1.1, 0.0);
		// Draw z axis:
		glVertex3f( 0.0, 0.0, -0.1);
		glVertex3f( 0.0, 0.0,  1.1);
	glEnd();
	
	swapBuffers();
}

void Canvas::initializeGL( ) 
{
	glClearColor (0.0, 0.0, 0.0, 0.0);	// Background to white

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
	glEnable (GL_DEPTH_TEST);
	glShadeModel( GL_SMOOTH );
}

void Canvas::InitMaterials(void)
{
	static float ambient[] = {0.1, 0.1, 0.1, 1.0};
	static float diffuse[] = {1.0, 1.0, 1.0, 1.0};


	static float front_mat_shininess[] = {50.0};
	static float front_mat_specular[] = {1.0, 1.0, 1.0, 1.0};
	static float front_mat_diffuse[] = {1.0, 1.0, 1.0, 1.0};


	static float lmodel_ambient[] = {1.0, 1.0, 1.0, 1.0};
	static float lmodel_twoside[] = {GL_FALSE};

	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);


	glEnable(GL_LIGHT0);

	glLightfv(GL_LIGHT1, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse);
	glEnable(GL_LIGHT1);

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);

	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_mat_shininess);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, front_mat_diffuse);


}

void Canvas::keyPressEvent ( QKeyEvent * e )
{
	switch ( tolower(e->ascii()) ) {
		case '+':
			scale *= 1.2;
			break;
		case '-':
			scale *= 0.8;
			break;
		case 'l':
			wholight = (++wholight)%2;
			break;
	}
	updateGL( );
}

